/*** Global Settings ****/
body {font : normal 12px/1.6em Arial, Helvetica, sans-serif;color : #2a3845;margin : 0;padding : 0;background:#FFFFFF;}
p { margin : 0;padding : 0;}
a:visited, a:link {color : #a43b55;text-decoration : none;background:none;}
a:hover {color : #a43b55;text-decoration : none;background:none;}
h1 { color : #7a2e40;margin: 0px 0px 10px 0px;padding-bottom : 10px;font : normal 17px Georgia, serif;border-bottom : 1px solid #efece7;}
h2 {color : #7a2e40;margin : 20px 0 10px;padding-bottom : 10px;font : normal 17px Georgia, serif;border-bottom : 1px solid #efece7;}
h3 {color : #7a2e40;margin : 10px 0;padding-bottom : 10px;font : bold 14px Arial, Helvetica, sans-serif;border-bottom : 1px solid #efece7;}
ul {padding:0;margin:0 0 0 17px;list-style: square url(images/bullet.gif);}
.box {background : #f7f7f7;border : 1px solid #f0e9eb;padding : 15px;}

/*** Layout ****/
#wrapper {margin : 0 auto;width :900px;border-left : 1px solid #f0e9eb;border-right : 1px solid #f0e9eb;}
#header {margin : 0 0px;}
#bg {height : 36px;background : url(images/bg.gif) repeat-x;}
#nav {background : #7a2e40;padding : 10px 20px;text-align : right;color : #f6dde3;}
#nav a:visited, #nav a:link {text-decoration : none;color : #f6dde3;}
#nav a:hover {text-decoration : underline;color : #f6dde3;}
#main-content {margin : 0 auto;}
#logo {margin-bottom : 45px;}
.logotxt1 {font : normal 28px Georgia, serif;color : #fa7393;margin : 10px 0 0 15px;}
.logotxt2 {font : normal 18px Georgia, serif;color : #7a2e40; }
#left-column {width : 440px;padding : 0 30px 30px;float : left;}
#main-column {width : 700px;padding : 0 30px 30px;float : left;}
#right-column {width : 270px;float : right;}
.crd a:visited, .crd a:link, .crd a:hover{text-decoration : none;color : #2a3845;cursor: text;}
#footer {background : #f7f7f7;border-top : 1px solid #f0e9eb;padding : 10px 15px;clear : both;}

/*** SideBAR ***/
#main-image {text-align : center;}
.sidebar {width : 218px;margin : 0 auto;padding : 10px 25px;background : url(images/shadow.jpg) no-repeat top;}

